#include <stdio.h>
#include <mkl.h>    // for BLAS & LAPACK
#include <math.h>

#ifndef M_PI
#define M_PI (4 * atan(1.0))
#endif

int main(void)
{
    for (int k = 1; k <= 14; ++k) {
        int N = 1 << k;
        int M = N - 1;
        double dx = M_PI / N;
        double *A, *u, *s;
        int *ip;

        A  = (double *)malloc(sizeof(double) * M * M);
        u  = (double *)malloc(sizeof(double) * M);
        s  = (double *)malloc(sizeof(double) * M);
        ip = (int *)   malloc(sizeof(int)    * M);

        // inhomogeneous term
        for (int n = 0; n < M; ++n) {
            u[n] = sin((n + 1) * dx);
        }
        cblas_dcopy(M, u, 1, s, 1);

        // coefficient matrix
        for (int m = 0; m < M; ++m) {
            for (int n = 0; n < M; ++n) {
                A[m * M + n]
                    = (m == n)      ?  2.0 / dx / dx :
                      (abs(m-n)==1) ? -1.0 / dx / dx :
                                       0.0           ;
            }
        }

        int info = LAPACKE_dgesv(LAPACK_COL_MAJOR,
                                    M, 1, A, M, ip, u, M);
        // Error estimates
        cblas_daxpy(M, -1.0, u, 1, s, 1);
        printf("%e %e %d\n",
                dx,
                fabs(s[cblas_idamax(M,s,1)]),
                info);

        free(A);
        free(u);
        free(s);
        free(ip);
    }
    return 0;
}
