#include <stdio.h>
#include <mkl.h>
#include <math.h>

#ifndef M_PI
#define M_PI (4 * atan(1.0))
#endif

int main(void)
{
	for (int k = 1; k <= 14; ++k) {
		int N = 1 << k;
		int M = N - 1;
		double *D, *E, *u, *s;
		double dx = M_PI / N;

		D = (double *)malloc(sizeof(double) * M);
		E = (double *)malloc(sizeof(double) * M);
		u = (double *)malloc(sizeof(double) * M);
		s = (double *)malloc(sizeof(double) * M);

		// inhomogeneous term
		for (int n = 0; n < M; ++n) {
			u[n] = sin((n + 1) * dx);
		}
		cblas_dcopy(M, u, 1, s, 1);

		// coefficient matrix
		for (int m = 0; m < M; ++m) {
			D[m] =  2.0 / dx / dx;
			E[m] = -1.0 / dx / dx;
		}

		int info = LAPACKE_dptsv(LAPACK_COL_MAJOR,
								 M, 1, D, E, u, M);

		// Error estimates
		cblas_daxpy(M, -1.0, u, 1, s, 1);
		printf("%e %e %d\n",
				dx, fabs(s[cblas_idamax(M,s,1)]), info);

		free(D);
		free(E);
		free(u);
		free(s);
	}
	return 0;
}
